

npops <- function(migmattext) {
   b <- unlist(strsplit(migmattext,"\\("))
   if (b[1]=="Island")
     return(as.integer(unlist(strsplit(b[2],","))[1]))
   if (b[1]=="General") {
     n <- length(unlist(strsplit(b[2],",")))
     rn <- floor(sqrt(n))
     if (n!=rn^2)
       stop("A general model should give a square matrix")
     return(rn);
   }
   stop(paste("'",b[1],"'"," not recognised as a migration type",sep=""))
}

makemigmatrix <- function(type="Island",mig=1,npops=3) {
  paste(type,"(",npops,",",mig,")")
}


"simARG" <- function(ss,sites,rec,growthmodel="constant"
                     ,migmatrix,poptree)
{
  ## void simplesim(int *ss, int *sites, double *rec, int *seed, int *count)

  
  if (length(rec)==1) r <- rep(rec,sites-1)
 
  if (missing(migmatrix)&&missing(poptree)) {
    wh <- .C("simplesim"
            ,as.integer(ss)
            ,as.integer(sites)
            ,as.double(r)
            ,as.character(growthmodel)
            ,as.integer(1)
             ,PACKAGE="ARG"
            )[[5]]
    r <- list(ss=ss,sites=sites,rec=rec,which=wh)
    class(r) <- "ARG"
    return(r)
  } else if (missing(poptree)) {
    if (length(ss)==1) {
      warning("using same sample size for each population")
      ss <- rep(ss,npops(migmatrix))
    }
    if (length(ss)!=npops(migmatrix)) 
      stop("Length of ss not the same as the number of populations")
    
    loc <- rep(1:length(ss),ss)-1
##    print(loc)
    wh <- .C("mmsim"
       ,as.integer(loc)
       ,as.integer(length(loc))
       ,as.integer(sites)
       ,as.double(r)
       ,as.character(growthmodel)
       ,as.character(migmatrix)
       ,as.integer(1)
             ,PACKAGE="ARG"
       )[[7]]
 
    r <- list(ss=ss,sites=sites,rec=rec,which=wh,location=loc+1
              ,growthmodel=growthmodel,migmatrix=migmatrix)
    class(r) <- c("structuredARG","ARG")
    return(r)
  } else if (missing(migmatrix)) {
    stop("poptree not defined yet")
  } else {
    stop("poptree not defined yet")
  }
}
### function to extract the tree heights 
"treeheight" <- function(graph)
  {
    .C("treeheight",as.integer(graph$which),as.double(1:graph$sites)
                                                      , PACKAGE="ARG")[[2]]
  }


### function to extract the tree lengths
"treelength" <- function(graph)
  {
    .C("treelength",as.integer(graph$which),as.double(1:graph$sites)
       ,PACKAGE="ARG")[[2]]
  }


"tree" <-  function(graph,position) {
  if (position<1||position>graph$sites) {
    stop("position should be between 1 and sites")
  }
  nchars=8192;
  txt <- .C("extracttree"
            ,as.integer(graph$which)
            ,as.integer(position-1)
            ,as.character(character(nchars))
            ,as.integer(nchars)
            ,PACKAGE="ARG")[[3]];
  
  read.tree(text=txt)
}



"mutate" <- function(graph,var,ascert="panel(3)")
  {
    if (var>0.1*graph$sites||var<1) {
       stop("variable sites must be between 1 and 10% of available sites (but 1% better)")
    }
    ##  void mutate(int *which, int *var, int *data, int *u, char **ascertainment)
    r <- .C("mutate"
            ,as.integer(graph$which)
            ,as.integer(var)
            ,as.integer(numeric(var*sum(graph$ss)))
            ,as.integer(numeric(var))
            ,as.character(ascert)
            ,PACKAGE="ARG");
    
    r <- list(ss=graph$ss,sites=graph$sites
              ,rec=graph$rec
              ,d= matrix(r[[3]],ncol=var,byrow=TRUE)
              ,positions=r[[4]]+1
              ,which=graph$which
              ,var=var);
    if (inherits(graph,"structuredARG")) {
      r$location <- graph$location
      class(r) <- c("mutatedstructuredARG","mutatedARG","structuredARG","ARG","GenomicHaplotype")
    } else {
    class(r) <- c("mutatedARG","ARG","GenomicHaplotype")
  }
    r
  }



 "prune" <- function(graph,keep)
  {
    if (length(keep)>graph$ss||length(keep)<2)
       stop("array of samples to keep must have length > 2 and less than sample size")

    r <- range(keep);
    if (r[1]<1||r[2]>graph$ss)
      stop("value to keep must be between 1 and sample size");

    ukeep <- unique(keep)
    if (length(ukeep)<length(keep))
      warning("repeated values in keep array, removed and continuing")
    
    .C("pruneARG"
       ,as.integer(graph$which)
       ,as.integer(keep-1)
       ,as.integer(length(keep))
       ,PACKAGE="ARG")
    graph$keep=ukeep
    graph;
  }


### what are the lengths of haplotypes around position
"haplengths" <- function(graph,position)
  {

    l <- .C("haplengths"
            ,as.integer(graph$which)
            ,as.integer(position-1)
            ,as.double(numeric(2*graph$ss))
            ,PACKAGE="ARG")[[3]]
    matrix(l,nrow=graph$ss)
  }

###
"sharedsectionARG" <- function(graph,position)
  {
    if (is.null(graph$keep))
      wh <- 0:(graph$ss-1)
    else wh <- graph$keep-1;
    
    l <- .C("sharedsection"
            ,as.integer(graph$which)
            ,as.integer(wh)
            ,as.integer(length(wh))
            ,as.integer(position-1)
            ,as.double(numeric(2*length(wh)))
            ,PACKAGE="ARG")[[5]]+1
    matrix(l,nrow=length(wh))
  }

###
"ARG.remove" <- function(graph)
  {
    .C("removeARG"
       ,as.integer(graph$which)
       ,PACKAGE="ARG")
  }
    
